//SlotMachine.java
//Carlos O. Rios
//Collaboration: I worked on the homework assignment alone, using only course materials.

import javax.swing.*;
import javax.swing.JOptionPane;
import java.awt.*;
import java.awt.event.*;

/**
* Creates the user interface for the slot machine. 
* Uses a BorderLayout, panels, and various graphical objects.
* Associates listeners for graphical objects. 
* @author Carlos O. Rios
* @version 1.0
*/

public class GamePanel extends JPanel
{
	/**
	*@param currentholdings is the current value of the winnings
	*/
	int currentholdings=100;
	
	/**
	*@param playvalue is the size of the bet
	*/
	int playvalue=1;
	
	/**
   *@JTextField playvaluetextfield is a textfield to obtain the size of the bet
	*/
	JTextField playvaluetextfield;
	
	/**
	*@JLabel playvaluetext is a label that contains the current size of the bet
	*/
	JLabel playvaluetext;
	
	/**
	*@JLabel currentholdingslabel is a label that contains the value of the users current winnings
	*/
	JLabel currentholdingslabel;
	
	/**
	*@JButton reset is a button that resets the current winnings to 100
	*/
	JButton reset;
	
	/**
	*@JRadioButton test places the game in test mode - grape, bell, and cherry are same probability
	*/
	JRadioButton test;
	
	/**
	*@JRadioButton play places the game in normal play mode where grape, bell, cherry, and lines are possible places
	*/
	JRadioButton play;
	
	/**
	*@JButton spin executes one spin
	*/
	JButton spin;
	
	/**
	*@JLabel leftdrum displays the value of the left drum after a spin
	*/
	JLabel leftdrum;
	
	/**
	*@JLabel middledrum displays the value of the middle drum after a spin
	*/
	JLabel middledrum;
	
	/**
	*@JLabel rightdrum displays the value of the right drum after a spin
	*/
	JLabel rightdrum;
	
	/**
	*@string mode contains a string representing the current mode of play - either "test" or "play"
	*/
	String mode;


	/**
	* This creates the GUI for the slot machine
	* It utilizes BorderLayout, panels, and various graphic objects
	* Associates listeners for graphic objects
	*/
	public SlotMachinePanel()
	{ 	
		setLayout( new BorderLayout());
		setBackground(Color.blue);
		
		/**
		* This creates the graphic components (buttons, text, etc.)
		*/
		String leftdrumstring="cherry";
		String middledrumstring="cherry";
		String rightdrumstring="cherry";
		JLabel machinename= new JLabel("SLOT MACHINE", JLabel.CENTER);
		leftdrum= new JLabel(leftdrumstring);
		middledrum= new JLabel(middledrumstring);
		rightdrum= new JLabel(rightdrumstring);
		JLabel currentholdingstext= new JLabel("CURRENT HOLDINGS",JLabel.RIGHT);
		playvaluetext= new JLabel("BET FOR THIS PLAY = 1 ",JLabel.RIGHT);
		test = new JRadioButton("TEST",false);
		play = new JRadioButton("PLAY",true);
		spin = new JButton("SPIN");
		reset = new JButton("RESET");
		currentholdingslabel= new JLabel("100",JLabel.RIGHT);
		playvaluetextfield= new JTextField("1",20);
		mode="play";
		
		
		/**
		* This creates three panels to insert in the center border layout 
		* Each will contain one drum that will correspond to it
		*/	
		JPanel drumpanelleft= new JPanel();
		drumpanelleft.setPreferredSize( new Dimension(300,50));
		drumpanelleft.add(leftdrum);
		
		JPanel drumpanelcenter= new JPanel();
		drumpanelcenter.setPreferredSize( new Dimension(300,50));
		drumpanelcenter.add(middledrum);
		
		JPanel drumpanelright= new JPanel();
		drumpanelright.setPreferredSize( new Dimension(300,50));
		drumpanelright.add(rightdrum);
		
		/**
		* This adds the three drum panels into a contained panel
		*/
		
		JPanel drumpanel= new JPanel();
		drumpanel.setLayout( new BoxLayout(drumpanel,BoxLayout.X_AXIS));
		drumpanel.add(drumpanelleft);
		drumpanel.add(drumpanelcenter);
		drumpanel.add(drumpanelright);
		
		/**
		* This creates a button group to add the radio buttons
		*/
		ButtonGroup group= new ButtonGroup();
		group.add(test);
		group.add(play);
		
		/**
		* This creates a panel that will hold the four buttons
		* Buttons in the panel are: reset, test, play and spin
		*/
		
		JPanel buttonpanel = new JPanel();
		buttonpanel.setPreferredSize( new Dimension(400,50));
		buttonpanel.add(reset);
		buttonpanel.add(test);
		buttonpanel.add(play);
		buttonpanel.add(spin);
		
		/**
		* This creates a panel to hold the play value
		*/
		
		JPanel playvaluepanel = new JPanel();
		playvaluepanel.setPreferredSize( new Dimension(300,50));
		playvaluepanel.setLayout( new BoxLayout(playvaluepanel,BoxLayout.Y_AXIS));
		playvaluepanel.add(playvaluetext);
		playvaluetext.setPreferredSize(new Dimension(50,25));		
		playvaluepanel.add(playvaluetextfield);
		
		/**
		* This creates a panel to hold the current holdings value
		*/
		
		JPanel currentholdingspanel = new JPanel();
		currentholdingspanel.setPreferredSize( new Dimension(200,50));
		currentholdingspanel.setLayout( new BoxLayout(currentholdingspanel,BoxLayout.Y_AXIS));
		currentholdingspanel.add(currentholdingstext);		
		currentholdingspanel.add(currentholdingslabel);
		
		/**
		* This creates a containing panel for the above three panels
		*/
		
		JPanel inoutpanel= new JPanel();
		inoutpanel.setLayout( new BoxLayout(inoutpanel,BoxLayout.X_AXIS));
		inoutpanel.add(buttonpanel);
		inoutpanel.add(playvaluepanel);
		inoutpanel.add(currentholdingspanel);
		
		/**
		* This adds the various components to the Border Layout
		*/
		
		add(machinename,BorderLayout.NORTH);	
		add(drumpanel,BorderLayout.CENTER);
		add(inoutpanel,BorderLayout.SOUTH);
		
		/**
		* This adds the listeners
		*/
		
		playvaluetextfield.addActionListener(new PlayvalueListener());
		ModeListener modelisten =new ModeListener();
		play.addActionListener(modelisten);
		test.addActionListener(modelisten);
		reset.addActionListener(new ResetbuttonListener());
		spin.addActionListener(new SpinbuttonListener());
		
	}
	
	/**
	* This provides an action listener for the playvaluetextfield
	* Then it gets the value of the current bet
	* It then determines if the bet is within the bounds "1 < bet < current value" of the holdings
	*/
	private class PlayvalueListener implements ActionListener
	{
		public void actionPerformed (ActionEvent event)
		{
			int fieldvalue;
			String text=playvaluetextfield.getText();
			fieldvalue=Integer.parseInt(text);
			if(fieldvalue < 1)
			{JOptionPane.showMessageDialog(null,"The bet must be one or greater");}
			else
			{	if(fieldvalue > currentholdings)
				{JOptionPane.showMessageDialog(null,"The bet must be smaller than your current holdings");}
				else
				{	playvalue=fieldvalue;
					String textdisplayed= "Bet for this play= "+text;
					playvaluetext.setText(textdisplayed);
				}
				
				
				
			}
			

		}
	}
	
	/**
	* This provides an action listener for the spin button
	* It also creates a SlotGame object
	* It then determines the play mode, plays one instance of the game, and updates the winning variable
	*/
	private class SpinbuttonListener implements ActionListener
	{
		public void actionPerformed (ActionEvent event)
		{SlotGame game = new SlotGame(playvalue,currentholdings);
		
		if (currentholdings < 1)
		{JOptionPane.showMessageDialog(null,"You ran out of money! Reset to play again");}
		else
		{
			if (mode=="play")
			{game.play();}
			else
			{game.test();}
		
		
			currentholdings=game.getcurrentholdings();
			String currentholdingsstring= Integer.toString(currentholdings);
			currentholdingslabel.setText(currentholdingsstring);
			if (currentholdings < 1)
			{JOptionPane.showMessageDialog(null,"You ran out of money, reset to play again");}
			leftdrum.setText(game.getdrum1());
			middledrum.setText(game.getdrum2());
			rightdrum.setText(game.getdrum3());
		 
		}					
		}
	}
	
	/**
	* This provides an action listener for the radio buttons mode and sets the mode instance variable to "play" or "test".
	*/
	private class ModeListener implements ActionListener
	{
		public void actionPerformed (ActionEvent event)
		{
			Object source=event.getSource();
			if (source == play)
			{ mode="play";}
			else
			{mode="test";}
				
		}
	}
		
		/**
		* This provides an action listener for the reset button
		* It asks the user to confirm resetting and resets the winnings to 100 if confirmed.
		*/
		private class ResetbuttonListener implements ActionListener
		{
		public void actionPerformed (ActionEvent event)
		{
		int result;
		result=JOptionPane.showConfirmDialog(null,"Are you sure you want to reset your account to 100?");
		if(result==JOptionPane.YES_OPTION)
		{currentholdings=100;
		String currentholdingsstring= Integer.toString(currentholdings);
		currentholdingslabel.setText(currentholdingsstring);
		}					
		}
	}
	
}